@@ -1,38 +1,46 @@
-#' Estimate Intracluster Correlation coefficient (ICC) and it's confidence intervals
+#' Estimates Intracluster Correlation coefficients (ICC) and it's confidence intervals (CI)
 #'
-#' Estimates Intracluster Correlation Coefficients (ICC) in 14 different methods and it's confidence intervals in 4 different methods given the data on cluster labels and outcomes
+#' Estimates Intracluster Correlation coefficients (ICC) in 16 different methods and it's confidence intervals (CI) in 5 different methods given the data on cluster labels and outcomes
 #'
 #' @param cid Column name indicating cluster id in the dataframe \code{data}
 #' @param y Column name indicating binary response in the dataframe \code{data}
 #' @param data A dataframe containing \code{cid} and \code{y}
-#' @param method The method to be used to compute ICC. A single or multiple methods can be used together. By default, all 14 methods will be used. See Details section for more.
-#' @param ci.type Type of confidence interval to be computed. By default all 4 types will be reported. See Detail section for more
-#' @param alpha The alpha level to be used when computing confidence interval. Default is 0.05
-#' @param kappa Value of Kappa to be used in computing Stabilized ICC when the method \code{stab} is chosen. Default is 0.45
+#' @param method The method to be used to compute ICC. A single or multiple methods can be used at a time. By default, all 16 methods will be used. See Details for more.
+#' @param ci.type Type of confidence interval to be computed. By default all 5 types will be reported. See Details for more
+#' @param alpha The significance level to be used while computing confidence interval. Default value is 0.05
+#' @param kappa Value of Kappa to be used in computing Stabilized ICC when the method \code{stab} is chosen. Default value is 0.45
+#' @param nAGQ An integer scaler, as in \code{glmer} function of package \code{lme4}, denoting the number of points per axis for evaluating the adaptive Gauss-Hermite approximation to the log-likelihood.
+#' Used when the method \code{lin} is chosen. Default value is 1
+#' @param M Number of Monte Carlo replicates used in ICC computation method \code{sim}. Default is 1000
 #'
 #' @details If in the dataframe, the cluster id (\code{cid}) is not a factor, it will be changed to a factor and a warning message will be given
-#' @details If estimate of ICC in any method is outside the interval [0, 1], the estimate and corresponding confidence interval (if appropriate) and warning messages will be produced
-#' @details If the lower limit of any confidence interval is below 0 and upper limit is above 1, they will be replaced by 0 and 1 respectively and warning message will be produced
-#' @details The method \code{aov} computes the analysis of variance estimate of ICC. This estimator was originally proposed for continuous variables, but various authors (e.g. Elston, 1977) have suggested it's use for binary variables
-#' @details The method \code{aovs} gives estimate of ICC using a modification of analysis of variance technique (see Fleiss, 1981)
-#' @details The method \code{keq} computes moment estimate of ICC suggested by Kleinman (1973), uses equal weight, \eqn{w_{i} = 1/k} for each of \eqn{k} clusters
-#' @details The method \code{kpr} computes moment estimate of ICC suggested by Kleinman (1973), uses weights proportionto cluster size \eqn{w_{i} = n_{i}/N}
-#' @details The method \code{keqs} gives a modified moment estimate of ICC with equal weights (\code{keq}) (see Kleinman, 1973)
-#' @details The method \code{kprs} gives a modified moment estimate of ICC with weights proportional to cluster size (\code{kpr}) (see Kleinman, 1973)
-#' @details The method \code{stab} provides a stabilized estimate of ICC proposed by Tamura and Young (1987)
-#' @details The method \code{ub} computes moment estimate of ICC from an unbiased estimating equation (see Yamamoto and Yanagimoto, 1992)
-#' @details The method \code{fc} gives Fleiss-Cuzick estimate of ICC (see Fleiss and Cuzick, 1979)
-#' @details The method \code{mak} computes Mak's estimate of ICC (see Mak, 1988)
-#' @details The method \code{peq} computes weighted correlation estimate of ICC proposed by karlin, Cameron, and Williams (1981) using equal weight to every pair of observations
-#' @details The method \code{pgp} computes weighted correlation estimate of ICC proposed by karlin, Cameron, and Williams (1981) using equal weight to each cluster irrespective of size
-#' @details The method \code{ppr} computes weighted correlation estimate of ICC proposed by karlin, Cameron, and Williams (1981) by weighting each pair according to the total number of pairs in which the individuals appear
-#' @details The method \code{rm} estimates ICC using resampling method proposed by Chakraborty and Sen (2016)
-#' @details The CI type \code{aov} computes confidence interval for ICC using Simith's large sample approximation (see Smith, 1957)
-#' @details The CI type \code{fc} gives Fleiss-Cuzick confidence interval for ICC (see Fleiss and Cuzick, 1979)
-#' @details The CI type \code{peq} estimates confidenc einterval for ICC based on direct calculation of correlation between observations within clusters (see Wu, Crespi, and Wong, 2012)
-#' @details The CI type \code{rm} gives confidence interval for ICC using resampling method by Chakraborty and Sen (2016)
+#' @details If estimate of ICC in any method is outside the interval [0, 1], the estimate and corresponding confidence interval (if appropriate) will not be provided and warning messages will be produced
+#' @details If the lower limit of any confidence interval is below 0 and upper limit is above 1, they will be replaced by 0 and 1 respectively and a warning message will be produced
+#' @details Method \code{aov} computes the analysis of variance estimate of ICC. This estimator was originally proposed for continuous variables, but various authors (e.g. Elston, 1977) have suggested it's use for binary variables
+#' @details Method \code{aovs} gives estimate of ICC using a modification of analysis of variance technique (see Fleiss, 1981)
+#' @details Method \code{keq} computes moment estimate of ICC suggested by Kleinman (1973), uses equal weight \eqn{w_{i} = 1/k}, for each of \eqn{k} clusters
+#' @details Method \code{kpr} computes moment estimate of ICC suggested by Kleinman (1973), uses weights proportional to cluster size \eqn{w_{i} = n_{i}/N}
+#' @details Method \code{keqs} gives a modified moment estimate of ICC with equal weights (\code{keq}) (see Kleinman, 1973)
+#' @details Method \code{kprs} gives a modified moment estimate of ICC with weights proportional to cluster size (\code{kpr}) (see Kleinman, 1973)
+#' @details Method \code{stab} provides a stabilizd estimate of ICC proposed by Tamura and Young (1987)
+#' @details Method \code{ub} computes moment estimate of ICC from an unbiased estimating equation (see Yamamoto and Yanagimoto, 1992)
+#' @details Method \code{fc} gives Fleiss-Cuzick estimate of ICC (see Fleiss and Cuzick, 1979)
+#' @details Method \code{mak} computes Mak's estimate of ICC (see Mak, 1988)
+#' @details Method \code{peq} computes weighted correlation estimate of ICC proposed by Karlin, Cameron, and Williams (1981) using equal weight to every pair of observations
+#' @details Method \code{pgp} computes weighted correlation estimate of ICC proposed by Karlin, Cameron, and Williams (1981) using equal weight to each cluster irrespective of size
+#' @details Method \code{ppr} computes weighted correlation estimate of ICC proposed by Karlin, Cameron, and Williams (1981) by weighting each pair according to the total number of pairs in which the individuals appear
+#' @details Method \code{rm} estimates ICC using resampling method proposed by Chakraborty and Sen (2016)
+#' @details Method \code{lin} estimates ICC using model linearization proposed by Goldstein et al. (2002)
+#' @details Method \code{sim} estimates ICC using Monte Carlo simulation proposed by Goldstein et al. (2002)
+#' @details CI type \code{aov} computes confidence interval for ICC using Simith's large sample approximation (see Smith, 1957)
+#' @details CI type \code{wal} computes confidence interval for ICC using modified Wald test (see Zou and Donner, 2004).
+#' @details CI type \code{fc} gives Fleiss-Cuzick confidence interval for ICC (see Fleiss and Cuzick, 1979; and Zou and Donner, 2004)
+#' @details CI type \code{peq} estimates confidence interval for ICC based on direct calculation of correlation between observations within clusters (see Zou and Donner, 2004; and Wu, Crespi, and Wong, 2012)
+#' @details CI type \code{rm} gives confidence interval for ICC using resampling method by Chakraborty and Sen (2016)
+#'
 #'
 #' @author Akhtar Hossain \email{mhossain@email.sc.edu}
+#' @author Hirshikesh Chakraborty \email{rishi.c@duke.edu}
 #'
 #' @return
 #' \item{estimates}{A dataframe containing the name of methods used and corresponding estimates of Intracluster Correlation coefficients}
@@ -42,6 +50,7 @@
 #' @references Elston, R.C., Hill, W.G. and Smith, C., 1977. Query: Estimating" Heritability" of a dichotomous trait. Biometrics, 33(1), pp.231-236.
 #' @references Fleiss, J.L., Levin, B. and Paik, M.C., 2013. Statistical methods for rates and proportions. John Wiley & Sons.
 #' @references Fleiss, J.L. and Cuzick, J., 1979. The reliability of dichotomous judgments: Unequal numbers of judges per subject. Applied Psychological Measurement, 3(4), pp.537-542.
+#' @references Goldstein, H., Browne, W., Rasbash, J., 2002. Partitioning variation in multilevel models, Understanding Statistics: Statistical Issues in Psychology, Education, and the Social Sciences, 1 (4), pp.223-231.
 #' @references Karlin, S., Cameron, E.C. and Williams, P.T., 1981. Sibling and parent--offspring correlation estimation with variable family size. Proceedings of the National Academy of Sciences, 78(5), pp.2664-2668.
 #' @references Kleinman, J.C., 1973. Proportions with extraneous variance: single and independent samples. Journal of the American Statistical Association, 68(341), pp.46-54.
 #' @references Mak, T.K., 1988. Analysing intraclass correlation for dichotomous variables. Applied Statistics, pp.344-352.
@@ -49,6 +58,7 @@
 #' @references Tamura, R.N. and Young, S.S., 1987. A stabilized moment estimator for the beta-binomial distribution. Biometrics, pp.813-824.
 #' @references Wu, S., Crespi, C.M. and Wong, W.K., 2012. Comparison of methods for estimating the intraclass correlation coefficient for binary responses in cancer prevention cluster randomized trials. Contemporary clinical trials, 33(5), pp.869-880.
 #' @references Yamamoto, E. and Yanagimoto, T., 1992. Moment estimators for the beta-binomial distribution. Journal of applied statistics, 19(2), pp.273-283.
+#' @references Zou, G., Donner, A., 2004 Confidence interval estimation of the intraclass correlation coefficient for binary outcome data, Biometrics, 60(3), pp.807-811.
 #'
 #' @seealso \code{\link{rcbin}}
 #'
@@ -57,14 +67,16 @@
 #' iccbin(cid = cid, y = y, data = bccdata)
 #' iccbin(cid = cid, y = y, data = bccdata, method = c("aov", "fc"), ci.type = "fc")
 #'
-#' @importFrom stats aggregate na.omit qnorm
+#' @importFrom stats aggregate na.omit qnorm binomial var
 #'
 #' @export
-
+#'
 
 iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr", "keqs", "kprs", "stab", "ub", "fc", "mak",
-                                                   "peq", "pgp", "ppr", "rm"), ci.type = c("aov", "fc", "peq", "rm"), alpha = 0.05, kappa = 0.45){
+                                                   "peq", "pgp", "ppr", "rm", "lin", "sim"),
+                    ci.type = c("aov", "wal", "fc", "peq", "rm"), alpha = 0.05, kappa = 0.45, nAGQ = 1, M = 1000){
 
+  CALL <- match.call()
   ic <- list(cid = substitute(cid), y = substitute(y))
 
   if(is.character(ic$y)){
@@ -130,22 +142,23 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
   msb <- (1/(k - 1))*(sum(yisq/ni) - (1/N)*(sum(yi))^2)
   msw <- (1/(N - k))*(sum(yi) - sum(yisq/ni))
   rho.aov <- (msb - msw)/(msb + (n0 - 1)*msw)
+
   if("aov" %in% method){
-    meth <- c(meth, "Analysis of Variance Estimator")
+    meth <- c(meth, "ANOVA Estimate")
     if(rho.aov < 0 | rho.aov > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Analysis of Variance' method")
+      warning("ICC not estimable by 'ANOVA' method")
     } else{
       est <- c(est, rho.aov)
     }
   }
 
   if("aov" %in% ci.type){
-    ci.typ <- c(ci.typ, "Smith's large sample confidence interval")
+    ci.typ <- c(ci.typ, "Smith's Large Sample Confidence Interval")
     if(rho.aov < 0 | rho.aov > 1){
       lci <- c(lci, "-")
       uci <- c(uci, "-")
-      warning("Smith's large sample confidence interval for ICC is not estimable")
+      warning("Smith's Large Sample Confidence Interval for ICC is Not Estimable")
     } else{
       st0 <- 2*square(1 - rho.aov)/square(n0)
       st1 <- square(1 + rho.aov*(n0 - 1))/(N - k)
@@ -156,7 +169,33 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
       lci <- c(lci, ifelse(ci.smith.rho.aov[1] < 0, 0, ci.smith.rho.aov[1]))
       uci <- c(uci, ifelse(ci.smith.rho.aov[2] > 1, 1, ci.smith.rho.aov[2]))
       if(ci.smith.rho.aov[1] < 0 | ci.smith.rho.aov[2] > 1){
-        warning("One or both of 'Smith's' confidence limits fell outside of [0, 1]")
+        warning("One or Both of 'Smith's' Confidence Limits Fell Outside of [0, 1]")
+      }
+    }
+  }
+
+  if("wal" %in% ci.type){
+    ci.typ <- c(ci.typ, "Zou and Donner's Modified Wald Confidence Interval")
+    if(rho.aov < 0 | rho.aov > 1){
+      lci <- c(lci, "-")
+      uci <- c(uci, "-")
+      warning("Zou and Donner's Modified Wald Confidence for ICC is Not Estimable")
+    } else{
+      piio <- sum(yi)/N
+      lambda <- (N - k)*(N - 1 - n0*(k - 1))*rho.aov + N*(k - 1)*(n0 - 1)
+      t0.zd <- (((k - 1)*n0*N*(N - k))^2)/lambda^4
+      t1.zd <- 2*k + (1/(piio*(1 - piio)) - 6)*sum(1/ni)
+      t2.zd <- ((1/(piio*(1 - piio)) - 6)*sum(1/ni) - 2*N + 7*k - (8*(k^2))/N - (2*k*(1 - k/N))/(piio*(1 - piio)) +
+        (1/(piio*(1 - piio)) - 3)*sum(ni^2))*rho.aov
+      t3.zd <- ((N^2 - k^2)/(piio*(1 - piio)) - 2*N - k + (4*(k^2))/N +
+        (7 - 8*k/N - (2*(1 - k/N))/(piio*(1 - piio)))*sum(ni^2))*rho.aov^2
+      t4.zd <- (1/(piio*(1 - piio)) - 4)*(((N - k)/N)^2)*(sum(ni^2) - N)*rho.aov^3
+      var.zd.rho.aov <- t0.zd*(t1.zd + t2.zd + t3.zd + t4.zd)
+      ci.zd.rho.aov <- c(rho.aov - zalpha*sqrt(var.zd.rho.aov), rho.aov + zalpha*sqrt(var.zd.rho.aov))
+      lci <- c(lci, ifelse(ci.zd.rho.aov[1] < 0, 0, ci.zd.rho.aov[1]))
+      uci <- c(uci, ifelse(ci.zd.rho.aov[2] > 1, 1, ci.zd.rho.aov[2]))
+      if(ci.zd.rho.aov[1] < 0 | ci.zd.rho.aov[2] > 1){
+        warning("One or Both of 'Zou and Donner's Modified Wald' Confidence Limits Fell Outside of [0, 1]")
       }
     }
   }
@@ -164,7 +203,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
   # ::: Modified ANOVA (Fleiss, 1981) :::
 
   if("aovs" %in% method){
-    meth <- c(meth, "Modified Analysis of Variance Estimator (Fleiss, 1981)")
+    meth <- c(meth, "Modified ANOVA Estimate")
     n0 <- (1/(k - 1))*(N - sum((ni^2)/N))
     yi <- aggregate(y, by = list(cid), sum)[ , 2]
     yisq <- yi^2
@@ -173,7 +212,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     rho.aovs <- (msbs - msw)/(msbs + (n0 - 1)*msw)
     if(rho.aovs < 0 | rho.aovs > 1){
       est <- c(est, "-")
-      warning("ICC Not Estimable by 'Modified Analysis of Variance' method")
+      warning("ICC Not Estimable by 'Modified ANOVA' Method")
     } else{
       est <- c(est, rho.aovs)
     }
@@ -187,7 +226,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
 
   # First type; Kleinman (1973); Ridout, Dometrio et al. (1999)
   if("keq" %in% method){
-    meth <- c(meth, "Moment Estimator with Equal Weights")
+    meth <- c(meth, "Moment Estimate with Equal Weights")
     pii <- yi/ni
     wi <- rep(1/k, k)
     piw <- sum(wi*pii)
@@ -195,7 +234,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     rho.keq <- (sw - piw*(1 - piw)*sum(wi*(1 - wi)/ni))/(piw*(1 - piw)*(sum(wi*(1 - wi))) - sum(wi*(1 - wi)/ni))
     if(rho.keq < 0 | rho.keq > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Moment with Equal Weights' method")
+      warning("ICC Not Estimable by 'Moment with Equal Weights' Method")
     } else{
       est <- c(est, rho.keq)
     }
@@ -203,7 +242,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
 
   # Second type; Kleinman (1973); Ridout, Dometrio et al. (1999)
   if("kpr" %in% method){
-    meth <- c(meth, "Moment Estimator with Weights Proportional to Cluster Size")
+    meth <- c(meth, "Moment Estimate with Weights Proportional to Cluster Size")
     pii <- yi/ni
     wi <- ni/N
     piw <- sum(wi*pii)
@@ -211,7 +250,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     rho.kpr <- (sw - piw*(1 - piw)*sum(wi*(1 - wi)/ni))/(piw*(1 - piw)*(sum(wi*(1 - wi))) - sum(wi*(1 - wi)/ni))
     if(rho.kpr < 0 | rho.kpr > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Moment with Weights Proportional to Cluster Size' method")
+      warning("ICC Not Estimable by 'Moment with Weights Proportional to Cluster Size' Method")
     } else{
       est <- c(est, rho.kpr)
     }
@@ -219,7 +258,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
 
   # Third type; Kleinman (1973); Ridout, Dometrio et al. (1999)
   if("keqs" %in% method){
-    meth <- c(meth, "Modified Moment Estimator with Equal Weights")
+    meth <- c(meth, "Modified Moment Estimate with Equal Weights")
     pii <- yi/ni
     wi <- rep(1/k, k)
     piw <- sum(wi*pii)
@@ -228,7 +267,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     rho.keqs <- (swn - piw*(1 - piw)*sum(wi*(1 - wi)/ni))/(piw*(1 - piw)*(sum(wi*(1 - wi))) - sum(wi*(1 - wi)/ni))
     if(rho.keqs < 0 | rho.keqs > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Modified Moment with Equal Weights' method")
+      warning("ICC Not Estimable by 'Modified Moment with Equal Weights' Method")
     } else{
       est <- c(est, rho.keqs)
     }
@@ -236,7 +275,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
 
   # Fourth type; Kleinman (1973); Ridout, Dometrio et al. (1999)
   if("kprs" %in% method){
-    meth <- c(meth, "Modified Moment Estimator with Weights Proportional to Cluster Size")
+    meth <- c(meth, "Modified Moment Estimate with Weights Proportional to Cluster Size")
     pii <- yi/ni
     wi <- ni/N
     piw <- sum(wi*pii)
@@ -245,7 +284,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     rho.kprs <- (swn - piw*(1 - piw)*sum(wi*(1 - wi)/ni))/(piw*(1 - piw)*(sum(wi*(1 - wi))) - sum(wi*(1 - wi)/ni))
     if(rho.kprs < 0 | rho.kprs > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Modified Moment with Weights Proportional to Cluster Size' method")
+      warning("ICC Not Estimable by 'Modified Moment with Weights Proportional to Cluster Size' Method")
     } else{
       est <- c(est, rho.kprs)
     }
@@ -253,7 +292,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
 
   # Fifth type; Tamura & Young (1987)
   if("stab" %in% method){
-    meth <- c(meth, "Stabilized Moment Estimator (Tamura & Young, 1987)")
+    meth <- c(meth, "Stabilized Moment Estimate")
     n0 <- (1/(k - 1))*(N - sum((ni^2)/N))
     kappa = 0.45
     p <- sum(yi)/sum(ni)
@@ -262,7 +301,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     rho.stab <- (1/(n0 - 1))*((N*sw)/((k - 1)*p*(1 - p)) + kappa - 1)
     if(rho.stab < 0 | rho.stab > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Stabilized Moment' method")
+      warning("ICC Not Estimable by 'Stabilized Moment' Method")
     } else{
       est <- c(est, rho.stab)
     }
@@ -270,7 +309,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
 
   # Sixth type; Yamamoto & Yanagimoto (1992)
   if("ub" %in% method){
-    meth <- c(meth, "Moment Estimator from Unbiased Estimating Equation")
+    meth <- c(meth, "Moment Estimate from Unbiased Estimating Equation")
     n0 <- (1/(k - 1))*(N - sum((ni^2)/N))
     yi <- aggregate(y, by = list(cid), sum)[ , 2]
     yisq <- yi^2
@@ -278,7 +317,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     rho.ub <- 1 - (N*n0*(k - 1)*msw)/(sum(yi)*(n0*(k - 1) - sum(yi)) + sum(yisq))
     if(rho.ub < 0 | rho.ub > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Unbiased Estimating Equation' method")
+      warning("ICC Not Estimable by 'Unbiased Estimating Equation' Method")
     } else{
       est <- c(est, rho.ub)
     }
@@ -291,10 +330,10 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
   piio <- sum(yi)/sum(ni)
   rho.fc <- 1 - (1/((N - k)*piio*(1 - piio)))*sum(yi*(ni - yi)/ni)
   if("fc" %in% method){
-    meth <- c(meth, "Fleiss-Cuzick Kappa Type Estimator")
+    meth <- c(meth, "Fleiss-Cuzick Kappa Type Estimate")
     if(rho.fc < 0 | rho.fc > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Fleiss-Cuzick Kappa' method")
+      warning("ICC Not Estimable by 'Fleiss-Cuzick's Kappa' Method")
     } else{
       est <- c(est, rho.fc)
     }
@@ -305,7 +344,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     if(rho.fc < 0 | rho.fc > 1){
       lci <- c(lci, "-")
       uci <- c(uci, "-")
-      warning("Fleiss-Cuzick confidence interval for ICC is not estimable")
+      warning("Fleiss-Cuzick Confidence Interval for ICC is Not Estimable")
     } else{
     t0.fc <- 1 - rho.fc
     t1.fc <- (1/(piio*(1 - piio)) - 6)*(sum(1/ni)/(N - k)^2) + (2*N + 4*k - (k/(piio*(1 - piio))))*(k/(N*(N - k)^2)) +
@@ -316,21 +355,21 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     lci <- c(lci, ifelse(ci.rho.fc[1] < 0, 0, ci.rho.fc[1]))
     uci <- c(uci, ifelse(ci.rho.fc[2] > 1, 1, ci.rho.fc[2]))
     if(ci.rho.fc[1] < 0 | ci.rho.fc[2] > 1){
-      warning("One or both of 'Fleiss-Cuzick' Confidence limits fell outside of [0, 1]")
+      warning("One or Both of 'Fleiss-Cuzick' Confidence Limits Fell Outside of [0, 1]")
     }
    }
   }
 
   # Mak (1988) method
   if("mak" %in% method){
-    meth <- c(meth, "Mak's Unweighted Average Estimator (Mak, 1988)")
+    meth <- c(meth, "Mak's Unweighted Average Estimate")
     yi <- aggregate(y, by = list(cid), sum)[ , 2]
     yisq <- yi^2
     ni <- as.vector(table(cid))
     rho.mak <- 1 - (k - 1)*sum((yi*(ni - yi))/(ni*(ni - 1)))/(sum(yisq/ni^2) + sum(yi/ni)*(k - 1 - sum(yi/ni)))
     if(rho.mak < 0 | rho.mak > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Mak's Unweighted' method")
+      warning("ICC Not Estimable by 'Mak's Unweighted' Method")
     } else{
       est <- c(est, rho.mak)
     }
@@ -344,10 +383,10 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
   mu.peq <- sum((ni - 1)*yi)/sum((ni - 1)*ni)
   rho.peq <- (1/(mu.peq*(1 - mu.peq)))*(sum(yi*(yi - 1))/sum(ni*(ni - 1)) - mu.peq^2)
   if("peq" %in% method){
-    meth <- c(meth, "Correlation Estimator with Equal Weight to Every Pair of Observations")
+    meth <- c(meth, "Correlation Estimate with Equal Weight to Every Pair of Observations")
     if(rho.peq < 0 | rho.peq > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Correlation Method with Weight to Every Pair of Observations'")
+      warning("ICC not Estimable by 'Correlation Method with Weight to Every Pair of Observations'")
     } else{
       est <- c(est, rho.peq)
     }
@@ -358,7 +397,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     if(rho.peq < 0 | rho.peq > 1){
       lci <- c(lci, "-")
       uci <- c(uci, "-")
-      warning("Pearson Correlation type confidence interval for ICC is not estimable")
+      warning("Pearson Correlation Type Confidence Interval for ICC is Not Estimable")
     } else{
     t0.peq <- (1 - rho.peq)/sum(ni*(ni - 1))^2
     t1.peq <- 2*sum(ni*(ni - 1)) + ((1/(piio*(1 - piio)) - 3)*sum(ni^2*(ni - 1)^2))*rho.peq
@@ -368,21 +407,21 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     lci <- c(lci, ifelse(ci.rho.peq[1] < 0, 0, ci.rho.peq[1]))
     uci <- c(uci, ifelse(ci.rho.peq[2] > 1, 1, ci.rho.peq[2]))
     if(ci.rho.peq[1] < 0 | ci.rho.peq[2] > 1){
-      warning("One or both of 'Pearson Correlation Type' Confidence limits fell outside of [0, 1]")
+      warning("One or Both of 'Pearson Correlation Type' Confidence Limits Fell Outside of [0, 1]")
     }
    }
   }
 
   # wi = 1/(k*ni*(ni - 1)) and equal weight for each group regardless of cluster size; Ridout, Demetrio, et al. (1999)
   if("pgp" %in% method){
-    meth <- c(meth, "Correlation Estimator with Equal Weight to Each Cluster Irrespective of Size")
+    meth <- c(meth, "Correlation Estimate with Equal Weight to Each Cluster Irrespective of Size")
     yi <- aggregate(y, by = list(cid), sum)[ , 2]
     ni <- as.vector(table(cid))
     mu.pgp <- sum(yi/ni)/k
     rho.pgp <- (1/(mu.pgp*(1 - mu.pgp)))*(sum((yi*(yi - 1))/(ni*(ni - 1)))/k - mu.pgp^2)
     if(rho.pgp < 0 | rho.pgp > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Correlation Method with Equal Weight to Each Cluster Irrespective of Size'")
+      warning("ICC Not Estimable by 'Correlation Method with Equal Weight to Each Cluster Irrespective of Size'")
     } else{
       est <- c(est, rho.pgp)
     }
@@ -390,14 +429,14 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
 
   # wi = 1/(k*ni*(ni - 1)) and equal weight for each group regardless of cluster size; Ridout, Demetrio, et al. (1999)
   if("ppr" %in% method){
-    meth <- c(meth, "Correlation Estimator with Weighting Each Pair According to Total Number of Pairs the Individuals Appear")
+    meth <- c(meth, "Correlation Estimate with Weighting Each Pair According to Number of Pairs individuals Appear")
     yi <- aggregate(y, by = list(cid), sum)[ , 2]
     ni <- as.vector(table(cid))
     mu.ppr <- sum(yi)/N
     rho.ppr <- (1/(mu.ppr*(1 - mu.ppr)))*(sum(yi*(yi - 1)/(ni - 1))/N - mu.ppr^2)
     if(rho.ppr < 0 | rho.ppr > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Correlation Method with Weighting Each Pair According to Total Number of Pairs the Individuals Appear'")
+      warning("ICC Not Estimable by 'Correlation Method with Weighting Each Pair According to Number of Pairs individuals Appear'")
     } else{
       est <- c(est, rho.ppr)
     }
@@ -410,7 +449,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
   n <- sum(ni)
   u1 <- sum(yi)/N
   alp <- u1
-  # Within cluster paire probabilities
+  # Within cluster pairwise probabilities
   ucid <- sort(unique(cid))
   nw11 <- 0; nw10 <- 0; nw01 <- 0; nw00 <- 0
   for(i in 1:k){
@@ -432,6 +471,8 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
       else{nw11 <- nw11 + 1}
     }
   }
+  # The frequencies are doubled as originiallny they were computed
+  # as half in above loop
   nw11n <- nw11*2; nw00n <- nw00*2
   nw10n <- nw10 + nw01; nw01n <- nw01 + nw10
   uw11 <- nw11n/sum(ni*(ni - 1))
@@ -440,7 +481,7 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
   uw00 <- nw00n/sum(ni*(ni - 1))
   tw <- uw11 + uw00 - uw10 - uw01
 
-  # Between cluster paire probabilities
+  # Between cluster pairwise probabilities
   nb11 <- 0; nb10 <- 0; nb01 <- 0; nb00 <- 0
   for(i in 1:(k - 1)){
     dti <- dt[cid == ucid[i], ]
@@ -460,6 +501,8 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
       }
     }
   }
+  # The frequencies are doubled as originiallny they were computed
+  # as half in above loop
   nb11n <- nb11*2; nb00n <- nb00*2
   nb10n <- nb10 + nb01; nb01n <- nb01 + nb10
   ub11 <- nb11n/(N*(N - 1) - sum(ni*(ni - 1)))
@@ -470,21 +513,21 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
 
   rho.rm <- (tw - tb)/(4*u1*(1 - u1))
   if("rm" %in% method){
-    meth <- c(meth, "Resampling Estimator (Chakraborty & Sen, 2016)")
+    meth <- c(meth, "Resampling Estimate")
     if(rho.rm < 0 | rho.rm > 1){
       est <- c(est, "-")
-      warning("ICC not estimable by 'Resampling' method")
+      warning("ICC Not Estimable by 'Resampling' Method")
     } else{
       est <- c(est, rho.rm)
     }
   }
 
   if("rm" %in% ci.type){
-    ci.typ <- c(ci.typ, "Resampling based Confidence Interval")
+    ci.typ <- c(ci.typ, "Resampling Based Confidence Interval")
     if(rho.rm < 0 | rho.rm > 1){
       lci <- c(lci, "-")
       uci <- c(uci, "-")
-      warning("Resampling based confidence interval for ICC is not estimable")
+      warning("Resampling Based Confidence Interval for ICC is Not Estimable")
     } else{
     t0.rm <- 1/(16*n*square(u1)*square(1 - u1))
     t1.rm <- 1/(n^2 - sum(ni^2)) + square(2*alp*(1 - alp) + 1)
@@ -496,11 +539,54 @@ iccbin <- function(cid, y, data = NULL, method = c("aov", "aovs", "keq", "kpr",
     lci <- c(lci, ifelse(ci.rho.rm[1] < 0, 0, ci.rho.rm[1]))
     uci <- c(uci, ifelse(ci.rho.rm[2] > 1, 1, ci.rho.rm[2]))
     if(ci.rho.rm[1] < 0 | ci.rho.rm[2] > 1){
-      warning("One or both of 'Resampling Based' Confidence limits fell outside of [0, 1]")
+      warning("One or Both of 'Resampling Based' Confidence Limits Fell Outside of [0, 1]")
     }
    }
   }
+
+  # Model Linearization and Monte Carlo Simulation Methods; Goldstein et al. (2002)
+  if("lin" %in% method | "sim" %in% method){
+    if(!requireNamespace("lme4", quietly = TRUE)){
+      stop("Package 'lme4' is needed for methods 'lin' and 'sim'. Please install it.",
+           call. = FALSE)
+    }
+    mmod <- lme4::glmer(y ~ 1 + (1 | cid), family = binomial, data = dt, nAGQ = nAGQ)
+    fint <- lme4::fixef(mmod)
+    re_var <- as.vector(lme4::VarCorr(mmod)[[1]])
+    if ("lin" %in% method){
+      meth <- c(meth, "First-order Model Linearized Estimate")
+      pr <- exp(fint)/(1 + exp(fint))
+      sig1 <- pr*(1 - pr)
+      sig2 <- re_var*pr^2*(1 + exp(fint))^(-2)
+      rho.lin <- sig2/(sig1 + sig2)
+      if(rho.lin < 0 | rho.lin > 1){
+        est <- c(est, "-")
+        warning("ICC Not Estimable by 'Model Linearization' Method")
+      } else{
+        est <- c(est, rho.lin)
+      }
+    }
+    if ("sim" %in% method){
+      meth <- c(meth, "Monte Carlo Simulation Estimate")
+      z <- rnorm(n = M, mean = 0, sd = sqrt(re_var))
+      pr <- exp(fint + z)/(1 + exp(fint + z))
+      sig1 <- mean(pr*(1 - pr))
+      sig2 <- var(pr)
+      rho.sim <- sig2/(sig1 + sig2)
+      if(rho.sim < 0 | rho.sim > 1){
+        est <- c(est, "-")
+        warning("ICC Not Estimable by 'Monte Carlo Simulation' Method")
+      } else{
+        est <- c(est, rho.sim)
+      }
+    }
+  }
+
   estimates <- data.frame(Methods = meth, ICC = est); row.names(estimates) <- NULL
   ci <- data.frame(Type = ci.typ, LowerCI = lci, UpperCI = uci); row.names(ci) <- NULL
   list(estimates = estimates, ci = ci)
+  #cat("\n", "ICC Estimates:", "\n"); print(estimates)
+  #cat("\n", paste(100*(1 - alpha), "%", sep = ""), "Confidence Intervals:", "\n"); print(ci)
 } # End of function ICCbin
+
+
